webnet = {}


function webnet:new(name,strict)
	local w = setmetatable({},{__index=webnet})
    if type(name) == 'string' then 
        local t,e = w:load(name,strict)
        if t == nil then w = nil; return nil,e end
    end
    return w
end

function webnet:load(name,strict)
	local last = -1
    local ib,ie,ia,_,co,cou,coun
    local ibn, ien
    local iptab,cotab = {},{}
    local f = io.open(name,"rt")
    

    if f == nil then return nil,"file not found" end
    

    for str in f:lines() do

        ib,ie,ia,_,co,cou,coun = 
                str:match('"(%d+),""(%d+)"",""(%l+)"",""(%d+)"",""(%u%u)"",""(%u%u%u?)"",""(.*)"""')

        if ib == nil or ie == nil or co == nil then
			
            
			
			if str:sub(1,1) == '"' then return nil,"bad line in file",str end
        else

            ibn, ien = tonumber(ib), tonumber(ie)
            if ibn <= last or ien < ibn then
			
			
                if strict then return nil,"ip data out of order" end
            else

                if ibn ~= last+1 then
                    iptab[#iptab+1] = last+1
                    cotab[#cotab+1] = "-- UNALLOCATED"
                end
				
				
				
                iptab[#iptab+1] = ibn
                cotab[#cotab+1] = co..' '..coun
                last = ien
            end
        end
    end    
    

    iptab[#iptab+1] = last+1
    cotab[#cotab+1] = "-- UNALLOCATED"
    
    self.iptab, self.cotab = iptab, cotab
    return self
end

function webnet:lookup(ip)

    if type(ip) == 'number' then 
       if ip<0 or ip>=2^32 then return nil,"bad ip number" end
    elseif type(ip) == 'string' then
       local a,b,c,d = ip:match('(%d+).(%d+).(%d+).(%d+)')
       if a==nil or b==nil or c==nil or d==nil then return nil, "bad ip string" end
       a,b,c,d = a+0,b+0,c+0,d+0
       if a<0 or b<0 or c<0 or d<0 then return nil,"bad ip string" end
       if a>255 or b>255 or c>255 or d>255 then return nil,"bad ip string" end
       ip = a*2^24 + b*2^16 + c*2^8 + d
    else
       return nil, "bad ip parameter"
    end
    

    local bot,top = 1,#self.iptab
    if ip <  self.iptab[bot] then return "--","UNALLOCATED" end
    if ip >= self.iptab[top] then return "--","UNALLOCATED" end
    
	
	
	local lap,lim = 0,(math.log(#self.iptab)/math.log(2))*1.5
    repeat
        lap = lap+1
        local mid = bot + math.floor((top-bot)/2)

        if ip >= self.iptab[mid] and ip < self.iptab[mid+1] then
            return self.cotab[mid]:match('(%C%C) (.*)')
        end

        if ip < self.iptab[mid] then top = mid else bot = mid+1 end
    until bot >= top or lap > lim
    

    return nil,"lookup error"
end
