local DIR = getnamedir
local NO_NAME = FireLangLua1
local ERROR_REPORTING = true
local MAX_AGE = 31
local NET_DELAY = 1500
local USGN_NAMES = {}





local _player = player
function player(id, t)
	if t == "usgnname" then return USGN_NAMES[id]
	else return _player(id, t) end
end




DIR = DIR:gsub("/", "\\")
if DIR:sub(-2) ~= "\\" then DIR = DIR .. "\\" end




local function _e (m) if ERROR_REPORTING and m then if type(m) == "string" then print("# GetUSGNName ERROR: " .. m) end end end



local function pipe(cmd)
	local f = assert(io.popen(cmd, "r"))
	local s = assert(f:read("*a"))
	f:close()
	if #s == 0 then _e("could not access " .. cmd .. " (in function pipe; while calling " .. cmd .. ")") return false end
	s = s:gsub("\n", "")
	return s
end



function GetUNameByID(id)
	if not id then _e("no player ID stated (in function GetUNameById)") return NO_NAME end
	if id <= 0 or id > 32 then _e("invalid player ID stated (in function GetUNameById)") return NO_NAME end
	
	local u = player(id, "usgn")
	if u == 0 then return NO_NAME end
	
	local name = pipe(DIR .. "GetUSGNName " .. u)
	if not name then return NO_NAME end
	if name:sub(1, 7) == "%ERROR%" then _e(name:sub(8) .. " (in function pipe; while calling GetUSGNName.py)") return NO_NAME end
	return name
end


function GetUNameByUSGN(usgn)
	if not usgn then _e("no U.S.G.N. ID # stated (in function GetUNameByUSGN)") return NO_NAME end
	if usgn <= 0 then _e("invalid U.S.G.N. ID # stated (in function GetUNameByUSGN)") return NO_NAME end	
	
	local name = pipe(DIR .. "GetUSGNName " .. usgn)
	if not name then return NO_NAME end
	if name:sub(1, 7) == "%ERROR%" then _e(name:sub(8) .. " (in function pipe; while calling GetUSGNName.py)") return NO_NAME end
	return name
end


local function SaveUNameById(id)
	if not id then _e("no player ID stated (in function SaveUNameById)") return NO_NAME end
	if id <= 0 or id > 32 then _e("invalid player ID stated (in function SaveUNameById)") return NO_NAME end
	
	local u = player(id, "usgn")
	if u == 0 then return NO_NAME end
	
	if not os.execute("start /min cmd /c " .. DIR .. "GetUSGNName " .. u .. " SAVE > nul") then _e("could not open process (in function SaveUNameById; while calling GetUSGNName.exe)") return false end
	return true
end





dhook("leave", "flua.cs.ClearUSGNName")

function flua.cs.SaveUSGNName(id)
	local u = player(id, "usgn")
	if not u then USGN_NAMES[id] = NO_NAME return 0 end
	
	local f = io.open(DIR .. "USGNNames/" .. u, "r")
	if f == nil then
		USGN_NAMES[id] = NO_NAME
		if SaveUNameById(id) == true then timer(NET_DELAY, "ReadUSGNName", u.."|"..id) end
	else
		local c = f:read("*all")
		local t = tonumber(c:sub(1, 10))
		f:close()
		if MAX_AGE > 0 and os.difftime(os.time(), t) >= MAX_AGE * 24 * 60 * 60 then
			os.remove(DIR .. "USGNNames/" .. u)
			USGN_NAMES[id] = NO_NAME
			if SaveUNameById(id) == true then timer(NET_DELAY, "ReadUSGNName", u.."|"..id) end
		else
			USGN_NAMES[id] = c:sub(11)
		end
	end
end


function ReadUSGNName(t)
	local u, id = t:sub(1,t:find("|")-1), tonumber(t:sub(t:find("|")+1))
	local f = io.open(DIR .. "USGNNames/" .. u, "r")
	if f ~= nil then
		local c = f:read("*all")
		USGN_NAMES[id] = c:sub(11)
		f:close()
	end
end


function flua.cs.ClearUSGNName(id)
	USGN_NAMES[id] = nil
end



for _, v in pairs(player(0, "table")) do flua.cs.SaveUSGNName(v) end